let Buffer = require('buffer').Buffer;

export class Sp8deCrypto {
    constructor(eth = undefined) {
        this.EthJS = !eth ? window.EthJS.Util : eth;
    }

    sing(privateKey, seed, nonce) {
        if (privateKey === undefined ||
            seed === undefined ||
            nonce === undefined) {
            console.error('Invalid parameters');
            return null;
        }
        let pubKey = this.EthJS.addHexPrefix(this.EthJS.privateToAddress(privateKey).toString('hex')),
            message = `${pubKey};${seed};${nonce}`,
            msg = this.EthJS.hashPersonalMessage(this.EthJS.toBuffer(message)),
            signed = this.EthJS.ecsign(msg, this.EthJS.toBuffer(privateKey)),
            tx = signed.r.toString('hex') + signed.s.toString('hex') + this.EthJS.stripHexPrefix(this.EthJS.intToHex(signed.v));
        return {
            pubKey: pubKey,
            message: message,
            sign: this.EthJS.addHexPrefix(tx),
            version: '3',
            signer: 'MEW'
        };
    }

    validate(signedMessage) {
        let item, hash, sign, pubKey;
        try {
            if (!signedMessage) throw new TypeError('Please enter signed message');
            item = typeof item === 'string' ? item = JSON.parse(signedMessage) : signedMessage;
            if (!item.sign) throw new TypeError('Empty sign');
            if (item.sign.length % 2 !== 0) throw new TypeError('Invalid hex string');
            sign = Buffer.from(this.getNakedAddress(item.sign), 'hex');
            if (sign.length !== 65) throw new TypeError('sign length is not valid');
            sign[64] = sign[64] === 0 || sign[64] === 1 ? sign[64] + 27 : sign[64];
            hash = this.EthJS.hashPersonalMessage(this.EthJS.toBuffer(item.message));
            if (item.version === '3') {
                if (item.signer === 'trezor') {
                    hash = this.getTrezorHash(item.message);
                } else if (item.signer === 'ledger') {
                    hash = this.EthJS.hashPersonalMessage(Buffer.from(item.message));
                }
            } else if (item.version === '1') {
                hash = this.EthJS.sha3(item.message);
            }
            pubKey = this.EthJS.ecrecover(
                hash,
                sign[64],
                sign.slice(0, 32),
                sign.slice(32, 64)
            );

            if (this.getNakedAddress(item.pubKey) !== this.EthJS.pubToAddress(pubKey).toString('hex')) throw new TypeError('sign is not valid');
        } catch (e) {
            if (e instanceof SyntaxError) console.error('JSON is not valid');
            else console.error(e);
            return false;
        }
        return true;
    }

    getNakedAddress(address) {
        return address.toLowerCase().replace('0x', '');
    };

    getTrezorLenBuf(msgLen) {
        if (msgLen < 253) return Buffer.from([msgLen & 0xff]);
        else if (msgLen < 0x10000)
            return Buffer.from([253, msgLen & 0xff, (msgLen >> 8) & 0xff]);
        else {
            return Buffer.from([
                254,
                msgLen & 0xff,
                (msgLen >> 8) & 0xff,
                (msgLen >> 16) & 0xff,
                (msgLen >> 24) & 0xff
            ]);
        }
    };

    getTrezorHash(msg) {
        return this.EthJS.sha3(
            Buffer.concat([
                this.EthJS.toBuffer('\u0019Ethereum Signed Message:\n'),
                this.getTrezorLenBuf(msg.length),
                this.EthJS.toBuffer(msg)
            ])
        );
    };

    generateSeed() {
        let rnd = new Uint32Array(1);
        window.crypto.getRandomValues(rnd);
        return rnd[0];
    };
}




